##############################################################################
#
# Copyright (c) 2007 Zope Foundation and Contributors.
# All Rights Reserved.
#
# This software is subject to the provisions of the Zope Public License,
# Version 2.1 (ZPL).  A copy of the ZPL should accompany this distribution.
# THIS SOFTWARE IS PROVIDED "AS IS" AND ANY AND ALL EXPRESS OR IMPLIED
# WARRANTIES ARE DISCLAIMED, INCLUDING, BUT NOT LIMITED TO, THE IMPLIED
# WARRANTIES OF TITLE, MERCHANTABILITY, AGAINST INFRINGEMENT, AND FITNESS
# FOR A PARTICULAR PURPOSE.
#
##############################################################################
import os
import unittest

from zope.testing.cleanup import CleanUp
import zope.configuration.xmlconfig

from z3c.pt import pagetemplate
from z3c.pt.pagetemplate import PageTemplateFile
from z3c.pt.pagetemplate import ViewPageTemplateFile

class Setup(CleanUp):

    def setUp(self):
        CleanUp.setUp(self)
        import z3c.pt
        zope.configuration.xmlconfig.file('configure.zcml', z3c.pt)

class TestPageTemplate(Setup,
                       unittest.TestCase):

    def test_string_function(self):
        template = pagetemplate.PageTemplate('''<div tal:content="python:string('a string')" />''')
        result = template.render()
        self.assertEqual(result,
                         '<div>a string</div>')

    def test_nocall_function(self):
        class Call(object):
            def __call__(self):
                raise AssertionError("Should not be called")
            def __str__(self):
                return "Not Called"
        arg = {'call': Call()}
        template = pagetemplate.PageTemplate('''<div tal:content="python:nocall('arg/call')" />''')
        result = template.render(arg=arg)
        self.assertEqual(result,
                         '<div>Not Called</div>')

class TestPageTemplateFile(Setup,
                           unittest.TestCase):

    def test_nocall(self):
        template = PageTemplateFile("nocall.pt")
        def dont_call():
            raise AssertionError("Should not be called")
        result = template(callable=dont_call)
        self.assertTrue(repr(dont_call) in result)

    def test_exists(self):
        template = PageTemplateFile("exists.pt")
        def dont_call():
            raise AssertionError("Should not be called")
        result = template(callable=dont_call)
        self.assertTrue('ok' in result)

    def test_false_attribute(self):
        template = PageTemplateFile("false.pt")
        result = template()
        self.assertTrue('False' in result)

    def test_boolean_attribute(self):
        template = PageTemplateFile("boolean.pt")
        result = template()
        self.assertFalse('False' in result)
        self.assertTrue('checked="checked"' in result)

    def test_path(self):
        template = PageTemplateFile("path.pt")

        class Context(object):
            dummy_wysiwyg_support = True

        context = Context()
        template = template.__get__(context, Context)

        result = template(editor="dummy")
        self.assertTrue("supported" in result)
        self.assertTrue("some path" in result)

class TestViewPageTemplateFile(Setup,
                               unittest.TestCase):

    def test_provider(self):

        class Context(object):
            pass

        class Request(object):
            response = None

        class View(object):
            __call__ = ViewPageTemplateFile("provider.pt")

        # Test binding descriptor behaviour.
        self.assertIsInstance(View.__call__, ViewPageTemplateFile)

        from zope.interface import Interface
        from zope.schema import Field
        from zope.interface import implementer
        from zope.interface import directlyProvides
        from zope.contentprovider.interfaces import ITALNamespaceData

        class ITestProvider(Interface):
            context = Field(u"Provider context.")

        directlyProvides(ITestProvider, ITALNamespaceData)
        assert ITALNamespaceData.providedBy(ITestProvider)

        @implementer(ITestProvider)
        class Provider(object):

            def __init__(self, *args):
                data.extend(list(args))

            def update(self):
                data.extend("updated")

            def render(self):
                return """<![CDATA[ %r, %r]]>""" % (
                    data, self.__dict__)

        view = View()
        data = []

        from zope.interface import implementedBy
        from zope.component import provideAdapter
        from zope.contentprovider.interfaces import IContentProvider

        provideAdapter(
            Provider, (
                implementedBy(Context),
                implementedBy(Request),
                implementedBy(View)
            ),
            IContentProvider,
            name="content"
        )

        context = Context()
        request = Request()

        result = view(context=context, request=request)
        self.assertIn(repr(data), result)
        self.assertIn(repr({'context': context}), result)

class TestOpaqueDict(unittest.TestCase):

    def test_getitem(self):
        import operator
        d = {}
        od = pagetemplate.OpaqueDict(d)
        with self.assertRaises(KeyError):
            operator.itemgetter('key')(od)

        d['key'] = 42
        self.assertEqual(od['key'], 42)

    def test_len(self):
        d = {}
        od = pagetemplate.OpaqueDict(d)
        self.assertEqual(0, len(od))

        d['key'] = 42
        self.assertEqual(1, len(od))

    def test_repr(self):
        d = {}
        od = pagetemplate.OpaqueDict(d)
        self.assertEqual('{...} (0 entries)', repr(od))

        d['key'] = 42
        self.assertEqual('{...} (1 entries)', repr(od))


class TestBaseTemplate(unittest.TestCase):

    def test_negotiate_fails(self):
        class I18N(object):
            request = None
            def negotiate(self, request):
                self.request = request
                raise Exception("This is caught")

        i18n = I18N()
        orig_i18n = pagetemplate.i18n
        pagetemplate.i18n = i18n
        try:
            template = pagetemplate.BaseTemplate('<html />')
            request = 'strings are allowed'
            template.render(request=request)
            self.assertIs(i18n.request, request)
        finally:
            pagetemplate.i18n = orig_i18n

    def test_translate_mv(self):
        template = pagetemplate.BaseTemplate("""
        <html>
          <body metal:use-macro="m" />
        </html>
        """)

        class Macro(object):
            translate = None
            def include(self, stream, econtext, *args, **kwargs):
                self.translate = econtext['translate']
        macro = Macro()
        template.render(m=macro)

        self.assertIsNone(macro.translate(pagetemplate.MV))

class TestBaseTemplateFile(unittest.TestCase):

    def test_init_with_path(self):

        here = os.path.abspath(os.path.dirname(__file__))

        template = pagetemplate.BaseTemplateFile('view.pt', path=here)

        self.assertEqual(template.filename,
                         os.path.join(here, 'view.pt'))

class TestBoundPageTemplate(unittest.TestCase):

    def test_setattr(self):
        bound = pagetemplate.BoundPageTemplate(None, None)
        with self.assertRaisesRegexp(AttributeError,
                                     "Can't set attribute"):
            setattr(bound, '__self__', 42)

    def test_repr(self):
        # It requires the 'filename' attribute
        class Template(object):
            filename = 'file.pt'

        bound = pagetemplate.BoundPageTemplate(Template(), 'render')
        self.assertEqual("<z3c.pt.tests.test_templates.BoundTemplate 'file.pt'>",
                         repr(bound))

    def test_attributes(self):
        func = object()
        bound = pagetemplate.BoundPageTemplate(self, func)
        self.assertIs(self, bound.im_self)
        self.assertIs(self, bound.__self__)
        self.assertIs(func, bound.im_func)
        self.assertIs(func, bound.__func__)



# -*- coding: utf-8 -*-
"""
Tests for expressions.py

"""
import unittest

from zope.testing.cleanup import CleanUp

from z3c.pt import expressions

# pylint:disable=protected-access

class TestRenderContentProvider(CleanUp,
                                unittest.TestCase):

    def test_not_found(self):
        from zope.contentprovider.interfaces import ContentProviderLookupError

        context = object()
        request = object()
        view = object()
        name = 'a provider'
        econtext = {'context': context, 'request': request, 'view': view}

        with self.assertRaises(ContentProviderLookupError) as exc:
            expressions.render_content_provider(econtext, name)

        e = exc.exception
        self.assertEqual(e.args,
                         (name, (context, request, view)))

    def test_sets_ilocation_name(self):
        from zope import component
        from zope import interface
        from zope.location.interfaces import ILocation
        from zope.contentprovider.interfaces import IContentProvider

        attrs = {}

        @interface.implementer(ILocation, IContentProvider)
        class Provider(object):
            def __init__(self, *args):
                pass

            def __setattr__(self, name, value):
                attrs[name] = value

            update = render = lambda s: None

        component.provideAdapter(Provider,
                                 adapts=(object, object, object),
                                 provides=IContentProvider,
                                 name='a provider')

        context = object()
        request = object()
        view = object()
        econtext = {'context': context, 'request': request, 'view': view}

        expressions.render_content_provider(econtext, 'a provider')

        self.assertEqual(attrs, {'__name__': 'a provider'})


class TestPathExpr(CleanUp,
                   unittest.TestCase):

    def test_translate_empty_string(self):
        import ast

        expr = expressions.PathExpr('')
        result = expr.translate('', 'foo')

        self.assertEqual(len(result), 1)
        self.assertIsInstance(result[0], ast.Assign)

    def test_translate_invalid_path(self):
        from chameleon.exc import ExpressionError

        expr = expressions.PathExpr('')
        with self.assertRaises(ExpressionError):
            expr.translate('not valid', None)

    def test_translate_components(self):
        from chameleon.codegen import TemplateCodeGenerator
        from chameleon.astutil import ASTCodeGenerator
        from chameleon.astutil import node_annotations
        expr = expressions.PathExpr('')
        comps = expr._find_translation_components(['a'])

        # First component is skipped
        self.assertEqual(comps, [])

        # Single literal
        comps = expr._find_translation_components(['a', 'b'])
        self.assertEqual([s.s for s in comps], ['b'])

        # Multiple literals
        comps = expr._find_translation_components(['a', 'b', 'c'])
        self.assertEqual([s.s for s in comps], ['b', 'c'])

        # Single interpolation---must be longer than one character
        comps = expr._find_translation_components(['a', '?b'])
        self.assertEqual([s.s for s in comps], ['?b'])

        comps = expr._find_translation_components(['a', '?var'])
        self.assertEqual(len(comps), 1)
        code = ASTCodeGenerator(comps[0]).code
        self.assertEqual(code, '(format % args)')
        code = TemplateCodeGenerator(comps[0]).code
        self.assertEqual(code, "('%s' % (var, ))")

        args = node_annotations[comps[0].right]
        code = TemplateCodeGenerator(args).code
        self.assertEqual(code, '(var, )')


        # Multiple interpolations
        comps = expr._find_translation_components(['a', '?var', "?var2"])
        self.assertEqual(len(comps), 2)
        code = ASTCodeGenerator(comps[0]).code
        self.assertEqual(code, '(format % args)')
        code = TemplateCodeGenerator(comps[0]).code
        self.assertEqual(code, "('%s' % (var, ))")

        args = node_annotations[comps[0].right]
        code = TemplateCodeGenerator(args).code
        self.assertEqual(code, '(var, )')

        code = ASTCodeGenerator(comps[1]).code
        self.assertEqual(code, '(format % args)')
        code = TemplateCodeGenerator(comps[1]).code
        self.assertEqual(code, "('%s' % (var2, ))")

        args = node_annotations[comps[1].right]
        code = TemplateCodeGenerator(args).code
        self.assertEqual(code, '(var2, )')

        translated = expr.translate("a/?var/?var2", None)
        self.assertEqual(len(translated), 1)
        code = TemplateCodeGenerator(translated[0]).code
        # XXX: Normally this starts with 'None =', but sometimes on Python 2,
        # at least in tox, it starts with '__package__ ='. Why
        # is this?
        code = code.strip().replace("__package__", 'None')
        self.assertEqual(
            code,
            "None = _path_traverse(a, econtext, True, (('%s' % (var, )), ('%s' % (var2, )), ))")



##############################################################################
#
# Copyright (c) 2007 Zope Foundation and Contributors.
# All Rights Reserved.
#
# This software is subject to the provisions of the Zope Public License,
# Version 2.1 (ZPL).  A copy of the ZPL should accompany this distribution.
# THIS SOFTWARE IS PROVIDED "AS IS" AND ANY AND ALL EXPRESS OR IMPLIED
# WARRANTIES ARE DISCLAIMED, INCLUDING, BUT NOT LIMITED TO, THE IMPLIED
# WARRANTIES OF TITLE, MERCHANTABILITY, AGAINST INFRINGEMENT, AND FITNESS
# FOR A PARTICULAR PURPOSE.
#
##############################################################################
import doctest
import unittest

import zope.component.testing
import zope.configuration.xmlconfig
import z3c.pt

OPTIONFLAGS = (doctest.ELLIPSIS
               | doctest.NORMALIZE_WHITESPACE)


def setUp(suite):
    zope.component.testing.setUp(suite)
    zope.configuration.xmlconfig.XMLConfig('configure.zcml', z3c.pt)()


def test_suite():
    filesuites = ('README.rst',)
    testsuites = 'z3c.pt.expressions', 'z3c.pt.namespaces'

    return unittest.TestSuite(
        [doctest.DocFileSuite(
            filesuite, optionflags=OPTIONFLAGS,
            setUp=setUp, tearDown=zope.component.testing.tearDown,
            package="z3c.pt")
         for filesuite in filesuites]
        +
        [doctest.DocTestSuite(
            testsuite, optionflags=OPTIONFLAGS,
            setUp=setUp, tearDown=zope.component.testing.tearDown)
         for testsuite in testsuites]
        )

if __name__ == '__main__':
    unittest.main(defaultTest='test_suite')



# -*- coding: utf-8 -*-
"""
Tests for namespaces.py.

"""

import unittest

from z3c.pt import namespaces

class TestAdapterNamespaces(unittest.TestCase):

    def test_registered_name(self):
        nss = namespaces.AdapterNamespaces()
        func = lambda ctx: ctx
        nss.registerFunctionNamespace('ns', func)
        self.assertIs(func, nss['ns'])
        self.assertIs(func, nss.getFunctionNamespace('ns'))

    def test_unregistered_name(self):
        nss = namespaces.AdapterNamespaces()
        with self.assertRaises(KeyError):
            nss.getFunctionNamespace('ns')

        # but __getitem__ makes one up
        self.assertIsNotNone(nss['ns'])

    def test_using_pagetemplate_version(self):
        from zope.pagetemplate import engine

        self.assertFalse(isinstance(namespaces.function_namespaces,
                                    namespaces.AdapterNamespaces))
        self.assertIsInstance(namespaces.function_namespaces,
                              engine.AdapterNamespaces)



##############################################################################
#
# Copyright (c) 2007 Zope Foundation and Contributors.
# All Rights Reserved.
#
# This software is subject to the provisions of the Zope Public License,
# Version 2.1 (ZPL).  A copy of the ZPL should accompany this distribution.
# THIS SOFTWARE IS PROVIDED "AS IS" AND ANY AND ALL EXPRESS OR IMPLIED
# WARRANTIES ARE DISCLAIMED, INCLUDING, BUT NOT LIMITED TO, THE IMPLIED
# WARRANTIES OF TITLE, MERCHANTABILITY, AGAINST INFRINGEMENT, AND FITNESS
# FOR A PARTICULAR PURPOSE.
#
##############################################################################
"""Setup
"""
import os
from setuptools import setup, find_packages


def read(*filenames):
    with open(os.path.join(*filenames)) as f:
        return f.read()


def alltests():
    import sys
    import unittest
    # use the zope.testrunner machinery to find all the
    # test suites we've put under ourselves
    import zope.testrunner.find
    import zope.testrunner.options
    here = os.path.abspath(os.path.join(os.path.dirname(__file__), 'src'))
    args = sys.argv[:]
    defaults = ["--test-path", here]
    options = zope.testrunner.options.get_options(args, defaults)
    suites = list(zope.testrunner.find.find_suites(options))
    return unittest.TestSuite(suites)

TESTS_REQUIRE = [
    'zope.pagetemplate',
    'zope.testing',
    'zope.testrunner',
]

setup(
    name='z3c.pt',
    version='3.1.0',
    author='Malthe Borch and the Zope Community',
    author_email='zope-dev@zope.org',
    description='Fast ZPT engine.',
    long_description=(
        read('README.rst')
        + '\n\n'
        + read('src', 'z3c', 'pt', 'README.rst')
        + '\n\n'
        + read('CHANGES.rst')
    ),
    license='ZPL',
    keywords='tal tales pagetemplate zope chameleon',
    classifiers=[
        'Development Status :: 5 - Production/Stable',
        'Environment :: Web Environment',
        'Intended Audience :: Developers',
        'License :: OSI Approved :: Zope Public License',
        'Programming Language :: Python',
        'Programming Language :: Python :: 2',
        'Programming Language :: Python :: 2.7',
        'Programming Language :: Python :: 3',
        'Programming Language :: Python :: 3.4',
        'Programming Language :: Python :: 3.5',
        'Programming Language :: Python :: 3.6',
        'Programming Language :: Python :: Implementation',
        'Programming Language :: Python :: Implementation :: CPython',
        'Programming Language :: Python :: Implementation :: PyPy',
        'Natural Language :: English',
        'Operating System :: OS Independent',
        'Topic :: Text Processing :: Markup :: HTML',
        'Topic :: Text Processing :: Markup :: XML',
        'Framework :: Zope3',
    ],
    url='https://github.com/zopefoundation/z3c.pt',
    namespace_packages=['z3c'],
    packages=find_packages('src'),
    package_dir={'': 'src'},
    install_requires=[
        'setuptools',
        'six',
        'zope.interface',
        'zope.component',
        'zope.i18n >= 3.5',
        'zope.traversing',
        'zope.contentprovider',
        'Chameleon >= 2.4',
    ],
    extras_require={
        'test': TESTS_REQUIRE,
    },
    tests_require=TESTS_REQUIRE,
    test_suite='__main__.alltests',
    include_package_data=True,
    zip_safe=False,
)
